function [Cmax, CmaxDelta, Cval_forET, C0s, C0Deltas] = Cmax(ts, L, omega0, maxET, C0min, C0max, C0step, critLFres_changethreshold, waitbarhandle)
%
%  Find Cmax by means of LF criterion of trend quality extraction
%

%
%  ts -- a time series (a row)
%  L -- window length
%  omega0 -- parameter of LF method/criterion (low frequencies boundary)
%  maxET -- amount of first ETs to be considered
%  [C0min, C0max] -- C0 interval
%  C0step 
%  critLFres_changethreshold -- difference between neigh. values of 
%                               LF criterion must overcome it 
%
%  
%  1. SSA
%  2. Calculate LowFreq values for all ETs
%  3. Change C0 from C0min to C0max with step C0step and identify
%     trend ETs with given C0
%  3.1. For each extracted trend calculate LF criterion values
%  4. Find Cmax -- C0 which corresponds to a minimal value of LF criterion
%
%
%  Theodore Alexandrov (autossa@gmail.com)
%  Last modified: Jun 18, 2006


showprogress = exist('waitbarhandle', 'var');

%  SSA
[sing_values,U,V] = ssa(ts, L);
d=length(sing_values);

if exist('maxET', 'var')
    d=min(d,maxET);
end


%  Calculate LowFreq values for all ETs
Cval_forET=zeros(d,1);
for et=1:d
   Cval_forET(et) = LFvalue_vect(U, omega0, et);
end


%  Change C0 from C0min to C0max with step C0step and 
%  identify trend ETs with given C0.
C0s_amount=floor( (C0max-C0min)/C0step )+1;


C0_ind=1;
crit_LFres=zeros(1, C0s_amount);
for C0=C0min:C0step:C0max
   %identify trend ETs
   j=1;
   trend_ETs=[];
   for et=1:d
       if  Cval_forET(et) > C0
           trend_ETs(j) = et;
           j=j+1;
       end
   end
   
   %reconstruct trend
   trend = ssa_reconstruct(sing_values, U, V, trend_ETs);

   %calculate LF criteria
   if C0>0
      crit_LFres(C0_ind) = LFCrit(ts, trend, omega0);
   else
      % It's neccessary because of excessive precision of Matlab calculations.
      % Because of this a time series which is the sum of all elementary 
      % time series isn't equal to an original time series.
      crit_LFres(C0_ind) = 0;
   end

   C0_ind=C0_ind+1;

   if showprogress
      waitbar((C0-C0min)/(C0max-C0min), waitbarhandle);
   end
end


C0s=C0min:C0step:C0max-C0step;
tmp(2:length(crit_LFres)+1)=crit_LFres;
C0Deltas=[crit_LFres, 0] - tmp;
C0Deltas=C0Deltas(2:length(C0Deltas)-1);

Cmax=1;
CmaxDelta=0;

for i=1:length(C0Deltas)
   if C0Deltas(i) > critLFres_changethreshold
      Cmax=C0s(i);       
      CmaxDelta=C0Deltas(i);
%      fprintf(1, 'C0max=%.3f, Delta LF residual(%.3f)=%.3f\n', C0s(i), C0s(i), C0Deltas(i));
      break; % because we want to find the minimal from such C0 values
   end
end

% tmpM=zeros(0);
% tmpM(1:length(dif), 1)=C0s(1:length(C0s)-1)';
% tmpM(1:length(dif), 2)=dif';
% sortedM=sortrows(tmpM, 2);
% h=size(sortedM, 1);
% fprintf(1, '3 max deltas: \n');
% fprintf(1, '\t delta(%.3f)=%.3f \n', sortedM(h-2:h, :)');


